define(function (require, exports, module) {
    var Anim = function (_shape, _duration, _callback) {
        var Animation = {},
            tweens = [],
            _tweens = {},
            options = {},
            attrs = _.extend({}, _shape.getAttrs());;


        options._duration = typeof _duration === 'undefined' ? 1 : _duration;
        options.node = _shape;

        // 清空/创建新tween
        var newTween = function () {
            _.size(_tweens) && tweens.push(_tweens);
            _tweens = {};
        }

        Animation.tween = function () {
            newTween();
            return Animation;
        }

        // 设置时间
        Animation.duration = function (_duration, _callback) {
            _tweens.duration = _duration * options._duration;
            return Animation;
        }

        // 设置角度
        Animation.rotation = function (_rotation) {
            _tweens.rotation = _rotation;
            return Animation;
        }

        // 设置透明度
        Animation.opacity = function (_opacity) {
            _tweens.opacity = _opacity;
            return Animation;
        }

        // 设置X轴坐标
        Animation.x = function (_x) {
            _tweens.x = _x;
            return Animation;
        }

        // 设置Y轴坐标
        Animation.y = function (_y) {
            _tweens.y = _y;
            return Animation;
        }

        // 设置过渡效果
        Animation.easing = function (_easing) {
            _tweens.easing = Konva.Easings[_easing];
            return Animation;
        }

        // 设置X轴拉伸倍数
        Animation.scaleX = function (_sx) {
            _tweens.scaleX = _sx;
            return Animation;
        }

        // 设置Y轴拉伸倍数
        Animation.scaleY = function (_sy) {
            _tweens.scaleY = _sy;
            return Animation;
        }

        // 执行动画
        var tweenPlay = function (_index) {
            if (!_.isUndefined(tweens[_index])) {
                new Konva.Tween(_.extend(tweens[_index], options, {
                    onFinish: function () {
                        tweenPlay(_index + 1);
                    }
                })).play()
            } else {
                if (attrs.opacity == 0) {
                    attrs.opacity = 1;
                }
                _shape.setAttrs(attrs);
                // console.log(_shape);
                typeof _callback === 'function' && _callback();
            }
        }

        Animation.play = function () {
            newTween();
            tweenPlay(0);
        }

        Animation.t = Animation.tween;
        Animation.d = Animation.duration;
        Animation.r = Animation.rotation;
        Animation.o = Animation.opacity;
        Animation.e = Animation.easing;
        Animation.sx = Animation.scaleX;
        Animation.sy = Animation.scaleY;

        return Animation;
    }

    var animList = {
        swing: swing,
        fadeOut: fadeOut,
        fadeIn: fadeIn,
        fadeInDown: fadeInDown,
        fadeInUp: fadeInUp,
        fadeInLeft: fadeInLeft,
        fadeInLeftBig: fadeInLeftBig,
        fadeInRight: fadeInRight,
        fadeInRightBig: fadeInRightBig,
        fadeOutDown: fadeOutDown,
        fadeOutUp: fadeOutUp,
        fadeOutLeft: fadeOutLeft,
        fadeOutRight: fadeOutRight,
        fadeOutLeftBig: fadeOutLeftBig,
        fadeOutRightBig: fadeOutRightBig,
        shake: shake,
        flash: flash,
        bounce: bounce,
        zoomIn: zoomIn,
        pull: pull,
        bounceInLeft: bounceInLeft,
        bounceInRight: bounceInRight,
        bounceInDown: bounceInDown,
        bounceInUp: bounceInUp,
        closer: closer,
        tada: tada
    }

    // 摇摆
    function swing(_shape, _duration, _callback) {
        var anim = new Anim(_shape, _duration, _callback);
        anim.tween().duration(.2).rotation(15);
        anim.tween().duration(.2).rotation(-10);
        anim.tween().duration(.2).rotation(5);
        anim.tween().duration(.2).rotation(-5);
        anim.tween().duration(.2).rotation(0);
        anim.play();
    }

    // 渐出
    function fadeOut(_shape, _duration, _callback) {
        var anim = new Anim(_shape, _duration, _callback);
        anim.tween().duration(1).opacity(0).play();
        return anim;
    }

    // 渐入
    function fadeIn(_shape, _duration, _callback) {
        var anim = new Anim(_shape, _duration, _callback);
        anim.tween().duration(0).opacity(0)
        anim.tween().duration(1).opacity(1).play();
        return anim;
    }

    // 从上往下渐入
    function fadeInDown(_shape, _duration, _callback) {
        var y = _shape.y(),
            anim = new Anim(_shape, _duration, _callback);
        anim.tween().duration(0).y(y - _shape.height()).opacity(0);
        anim.tween().duration(1).y(y).opacity(1).easing('EaseOut');
        anim.play();
    }

    // 从下往上渐入
    function fadeInUp(_shape, _duration, _callback) {
        var y = _shape.y(),
            anim = new Anim(_shape, _duration, _callback),
            opacity = _shape.opacity();
        anim.tween().duration(0).y(y + _shape.height()).opacity(0);
        anim.tween().duration(1).y(y).opacity(opacity ? opacity : 1).easing('EaseOut');
        anim.play();
    }

    // 从左往右渐入
    function fadeInLeft(_shape, _duration, _callback) {
        var x = _shape.x(),
            width = _shape.width(),
            anim = new Anim(_shape, _duration, _callback);
        anim.tween().duration(0).x(x - width).opacity(0);
        anim.tween().duration(1).x(x).opacity(1);
        anim.play();
    }

    // 从左往右渐入 BIG
    function fadeInLeftBig(_shape, _duration, _callback) {
        var x = _shape.x(),
            width = _shape.width(),
            anim = new Anim(_shape, _duration, _callback);
        anim.tween().duration(0).x(-2000).opacity(0);
        anim.tween().duration(1).x(x).opacity(1);
        anim.play();

    }

    // 从右往左渐入
    function fadeInRight(_shape, _duration, _callback) {
        var x = _shape.x(),
            width = _shape.width(),
            anim = new Anim(_shape, _duration, _callback);
        anim.tween().duration(0).x(x + width).opacity(0);
        anim.tween().duration(1).x(x).opacity(1);
        anim.play();
    }

    // 从右往左渐入 BIG
    function fadeInRightBig(_shape, _duration, _callback) {
        var x = _shape.x(),
            width = _shape.width(),
            anim = new Anim(_shape, _duration, _callback);
        anim.tween().duration(0).x(2000).opacity(0);
        anim.tween().duration(1).x(x).opacity(1);
        anim.play();
    }

    // 向下渐出
    function fadeOutDown(_shape, _duration, _callback) {
        var y = _shape.y(),
            anim = new Anim(_shape, _duration, _callback);
        anim.tween().duration(0).y(y).opacity(1);
        anim.tween().duration(1).y(y + _shape.height()).opacity(0).easing('EaseOut');
        anim.play();
    }

    // 向上渐出
    function fadeOutUp(_shape, _duration, _callback) {
        var y = _shape.y(),
            anim = new Anim(_shape, _duration, _callback);
        anim.tween().duration(0).y(y).opacity(1);
        anim.tween().duration(1).y(y - _shape.height()).opacity(0).easing('EaseOut');
        anim.play();
    }

    // 向左渐出
    function fadeOutLeft(_shape, _duration, _callback) {
        var x = _shape.x(),
            width = _shape.width(),
            anim = new Anim(_shape, _duration, _callback);
        anim.tween().duration(0).x(x).opacity(1);
        anim.tween().duration(1).x(x - width).opacity(0);
        anim.play();
    }

    // 向右渐出
    function fadeOutRight(_shape, _duration, _callback) {
        var x = _shape.x(),
            width = _shape.width(),
            anim = new Anim(_shape, _duration, _callback);
        anim.tween().duration(0).x(x).opacity(1);
        anim.tween().duration(1).x(x + width).opacity(0);
        anim.play();
    }

    function fadeOutRightBig(_shape, _duration, _callback) {
        var x = _shape.x(),
            width = _shape.width(),
            anim = new Anim(_shape, _duration, _callback);
        anim.tween().duration(0).x(x).opacity(1);
        anim.tween().duration(1).x(2000).opacity(0);
        anim.play();
    }

    function fadeOutLeftBig(_shape, _duration, _callback) {
        var x = _shape.x(),
            width = _shape.width(),
            anim = new Anim(_shape, _duration, _callback);
        anim.tween().duration(0).x(x).opacity(1);
        anim.tween().duration(1).x(x - 2000).opacity(0);
        anim.play();
    }



    // 摇晃
    function shake(_shape, _duration, _callback) {
        var x = _shape.x(),
            anim = new Anim(_shape, _duration, _callback);
        anim.tween().duration(0).x(x);
        anim.tween().duration(.1).x(x - 10);
        anim.tween().duration(.1).x(x + 10);
        anim.tween().duration(.1).x(x - 10);
        anim.tween().duration(.1).x(x + 10);
        anim.tween().duration(.1).x(x - 10);
        anim.tween().duration(.1).x(x + 10);
        anim.tween().duration(.1).x(x - 10);
        anim.tween().duration(.1).x(x + 10);
        anim.tween().duration(.1).x(x);
        anim.play();
    }

    // 闪烁
    function flash(_shape, _duration, _callback) {
        var anim = new Anim(_shape, _duration, _callback);
        anim.tween().duration(0).opacity(1);
        anim.tween().duration(.25).opacity(0);
        anim.tween().duration(.25).opacity(1);
        anim.tween().duration(.25).opacity(0);
        anim.tween().duration(0).opacity(1);
        anim.play();
    }

    // 弹跳
    function bounce(_shape, _duration, _callback) {
        var anim = new Anim(_shape, _duration, _callback),
            y = _shape.y();;
        anim.tween().y(y).duration(0).easing('EaseOut');
        anim.tween().y(y).duration(.2).easing('EaseOut');
        anim.tween().y(y - 30).duration(.2).easing('EaseOut');
        anim.tween().y(y - 30).duration(.03).easing('EaseIn');
        anim.tween().y(y).duration(.1).easing('EaseOut');
        anim.tween().y(y - 15).duration(.17).easing('EaseOut');
        anim.tween().y(y).duration(.1).easing('EaseIn');
        anim.tween().y(y - 4).duration(.1).easing('EaseOut');
        anim.tween().y(y).duration(.1).easing('EaseOut');
        anim.play();

    }

    // 放大
    function zoomIn(_shape, _duration, _callback) {
        var anim = new Anim(_shape, _duration, _callback);
        anim.tween().duration(0).opacity(.3).scaleX(.3).scaleY(.3);
        anim.tween().duration(1).opacity(1).scaleX(1).scaleY(1);
        anim.play();
    }

    // 从大变小
    function pull(_shape, _duration, _callback) {
        // 调整 offset
        _shape.offsetX(_shape.width() / 2);
        _shape.x(_shape.width() / 2);

        var anim = new Anim(_shape, _duration, _callback);
        anim.tween().duration(0).opacity(0).scaleX(1.5).scaleY(1.5);
        anim.tween().duration(.5).opacity(1);
        anim.tween().duration(.5).opacity(1).scaleX(1).scaleY(1)
        anim.play();
    }

    // 从左往右弹跳进入
    function bounceInLeft(_shape, _duration, _callback) {
        var anim = new Anim(_shape, _duration, _callback),
            x = _shape.x();
        anim.tween().duration(0).opacity(0).x(x - 3000).easing('EaseOut');
        anim.tween().duration(.6).opacity(1).x(x + 25).easing('EaseOut');
        anim.tween().duration(.15).x(x - 10).easing('EaseOut');
        anim.tween().duration(.15).x(x + 5).easing('EaseOut');
        anim.tween().duration(.1).x(x).easing('EaseOut');
        anim.play();
    }

    // 从右往左弹跳进入
    function bounceInRight(_shape, _duration, _callback) {
        var anim = new Anim(_shape, _duration, _callback),
            x = _shape.x();
        anim.tween().duration(0).opacity(0).x(x + 3000).easing('EaseOut');
        anim.tween().duration(.6).opacity(1).x(x - 25).easing('EaseOut');
        anim.tween().duration(.15).x(x + 10).easing('EaseOut');
        anim.tween().duration(.15).x(x - 5).easing('EaseOut');
        anim.tween().duration(.1).x(x).easing('EaseOut');
        anim.play();
    }

    // 从上往下弹跳进入
    function bounceInDown(_shape, _duration, _callback) {
        var anim = new Anim(_shape, _duration, _callback),
            y = _shape.y();
        anim.tween().duration(0).opacity(0).y(y - 3000).easing('EaseOut');
        anim.tween().duration(.6).opacity(1).y(y + 25).easing('EaseOut');
        anim.tween().duration(.15).y(y - 10).easing('EaseOut');
        anim.tween().duration(.15).y(y + 5).easing('EaseOut');
        anim.tween().duration(.1).y(y).easing('EaseOut');
        anim.play();
    }

    // 从下往上弹跳进入
    function bounceInUp(_shape, _duration, _callback) {
        _shape.opacity(0);
        var anim = new Anim(_shape, _duration, _callback),
            y = _shape.y();
        anim.tween().duration(0).opacity(0).y(y + 3000).easing('EaseOut');
        anim.tween().duration(.6).opacity(1).y(y - 25).easing('EaseOut');
        anim.tween().duration(.15).y(y + 10).easing('EaseOut');
        anim.tween().duration(.15).y(y - 5).easing('EaseOut');
        anim.tween().duration(.1).y(y).easing('EaseOut');
        anim.play();
    }

    //放大
    function closer(_shape, _duration, _callback) {
        var anim = new Anim(_shape, _duration, _callback);
        anim.tween().duration(0).opacity(0).scaleX(0).scaleY(0);
        anim.tween().duration(.45).opacity(.75).scaleX(.5).scaleY(.5);
        anim.tween().duration(.55).opacity(1).scaleX(1).scaleY(1);
        anim.play();
    }

    // tada 效果
    function tada(_shape, _duration, _callback) {
        //调整offset
        var x = _shape.x(),
            y = _shape.y();

        if (!_shape.offsetX()) {
            _shape.x(x + _shape.width() / 2)
            _shape.offsetX(_shape.width() / 2);
        }

        if (!_shape.offsetY()) {
            _shape.offsetY(_shape.height() / 2);
            _shape.y(y + _shape.height() / 2)
        }

        var anim = new Anim(_shape, _duration, _callback);

        anim.tween().duration(0).scaleX(1).scaleY(1);
        anim.tween().duration(.2).scaleX(.9).scaleY(.9).rotation(-3);
        anim.tween().duration(.1).scaleX(1.1).scaleY(1.1).rotation(3);
        anim.tween().duration(.1).scaleX(1.1).scaleY(1.1).rotation(-3);
        anim.tween().duration(.1).scaleX(1.1).scaleY(1.1).rotation(3);
        anim.tween().duration(.1).scaleX(1.1).scaleY(1.1).rotation(-3);
        anim.tween().duration(.1).scaleX(1.1).scaleY(1.1).rotation(3);
        anim.tween().duration(.1).scaleX(1.1).scaleY(1.1).rotation(-3);
        anim.tween().duration(.1).scaleX(1.1).scaleY(1.1).rotation(3);
        anim.tween().duration(.1).scaleX(1).scaleY(1).rotation(0);
        anim.play();
    }

    module.exports.play = function (_aniName, _shape, _duration, _callback, delay) {
        if (_aniName == 'undefined' || _aniName == '' || _aniName == 'noAnimate')
            return _shape.setAttr('opacity', 1);

        delay = (delay ? delay : 0) * 1000;
        setTimeout(function () {
            if (!_.isUndefined(animList[_aniName])) {
                return animList[_aniName].call(this, _shape, _duration, _callback)
            } else {
                console.error('对不起，动画【' + _aniName + '】未被支持', _shape);
            }
        }, delay)
    }

    // 播放一个动画列表
    module.exports.playList = function (_aniName, _shapeList, _duration, _callback, delay) {
        if (_aniName === 'undefined' || _aniName == '' || _aniName == 'noAnimate')
            return _.each(_shapeList, function (elem) {
                elem.setAttr('opacity', 1);
            });
        delay = (delay ? delay : 0) * 1000;
        setTimeout(function () {
            if (!_.isUndefined(animList[_aniName])) {
                return Thenjs.eachSeries(_shapeList, function (cont, value, key) {
                    animList[_aniName].call(this, value, _duration, function () {
                        cont();
                    })
                }).then(function () {
                    typeof _callback === 'function' && _callback();
                })
            } else {
                console.error('对不起，' + _aniName + ' 未被支持');
            }
        }, delay);
    }

    // 文本数组 显示速度 延迟事件 事件绑定对象 回调函数
    module.exports.typewriter = function (_shapeList, _speed, delay, _eventProxy, callback) {
        delay = (delay ? delay : 0) * 1000;
        setTimeout(function () {
            var index = 0,
                shape = {},
                typing_stop = false;
            var timeout = setTimeout(function next() {
                if (index >= _shapeList.length) {
                    !typing_stop && typeof callback === 'function' && callback();
                    return clearTimeout(timeout);
                }
                animList['fadeIn'].call(this, _shapeList[index++], _speed);
                timeout = setTimeout(next, _speed * 1000);
            }, _speed * 1000);

            _eventProxy.unbind('TYPING_STOP').on('TYPING_STOP', function (callback) {
                clearTimeout(timeout);
                typing_stop = true;
                _.each(_shapeList, function (elem) {
                    elem.opacity(1);
                });
                typeof callback === 'function' && callback();
            });

        }, delay);
    }
})
