// 文本模块
define(function(require, exports, module) {
    // 解析文字 
    // 参数 文本列表 容器宽度 单行文字数量
    var parseText = function(_textArray, _containerWidth, _fontBits, _align) {
        var _textDealArray = [],
            fontWidth = _containerWidth / _fontBits,
            fontHeight = fontWidth,
            counter = 0,
            line = 0,
            lineFonts = {};
        // 考虑极限值
        if (fontWidth < 12) {
            fontWidth = 12;
            _fontBits = Math.round(_containerWidth / fontWidth);
        }
        if (fontHeight < 12) fontHeight = 12;
        fontHeight += fontHeight / 3;

        // 封装文本
        _.each(_textArray, function(elem) {
            if (elem.newLine) {
                counter = 0;
                line++;
                return true;
            }
            // 解析文字属性
            var type = elem.type ? elem.type.split(',') : [],
                underline = false,
                _type = [];
            _.each(type, function(_elem) {
                _elem == 'B' && _type.push('bold');
                _elem == 'I' && _type.push('italic');
                if (_elem == 'U') underline = true;
            });

            for (var i = 0; i < elem.text.length; i++) {
                // 配置行数与文本序号
                if (counter + 1 > _fontBits) {
                    line++
                    counter = 0;
                }
                // 配置每行字数
                if (_.isUndefined(lineFonts[line]))
                    lineFonts[line] = 0;
                lineFonts[line]++;
                // 将初步配置的文本填充到临时处理数组
                _textDealArray.push(_.extend({
                    text: elem.text[i],
                    line: line,
                    index: counter,
                    width: fontWidth,
                    height: fontHeight
                }, {
                    fill: elem.color ? elem.color : '#ffffff',
                    fontSize: elem.fontSize > fontWidth ? fontWidth : (elem.fontSize ? elem.fontSize : 20),
                    fontStyle: _type.length ? _type.join(' ') : [],
                    underline: underline,
                    align: 'center'
                }));
                ++counter;
            }
        });
        // 调整文本位置
        _.each(_textDealArray, function(elem, key) {
            var diff = 0;
            switch (_align) {
                case 'center':
                    diff = (_containerWidth - fontWidth * lineFonts[elem.line]) / 2;
                    break;
                case 'right':
                    diff = (_containerWidth - fontWidth * lineFonts[elem.line]);
                    break;
            }
            _textDealArray[key].x = diff + fontWidth * elem.index
            _textDealArray[key].y = elem.line * fontHeight;
        });

        return _textDealArray;
    }

    function splitAndJoin(child) {
        var arr = child.html().split('</font>');
        _.each(arr, function(_childelem, key) {
            var _arr = _childelem.split('<font');
            if (_arr.length > 1) {
                if (_arr[0] != '') {
                    _arr[0] = '<font>' + _arr[0] + '</font>';
                }
                arr[key] = _arr.join('<font')
            } else {
                if (arr[key] != '') {
                    arr[key] = '<font>' + arr[key] + '</font>';
                } else {
                    delete arr[key]
                }
            }
        })
        child.html(arr.join('</font>'));
    }

    // 清理文本BIU类型的方法 常见可能为空或,或B,I,U等
    function cleanUpType(_type) {
        if (_type == '') return [];
        _type = _type.split(',');
    }

    module.exports.parseText = parseText;

    // 解析HTML成单位字
    // "Html": "<p><u>我们<b>都是</b><font color=\"#ff00ff\">好</font><font color=\"#ffff00\"><b>孩</b></font>子啊</u></p>"
    module.exports.parseHTML = function(_html, _containerWidth, _opts, _align) {
        if (_.isUndefined(_html)) return;
        var _fontSize = Math.ceil(_opts.fontSize);
        _.each(_opts.responsive, function(elem, key) {

            key = key.split('~');
            if ((key[0] == "" && _opts._width < key[1]) || (_opts._width < key[0] && _opts._width < key[1]) || (key[1] == '' && key[0] < _opts._width))
                _fontSize = elem.fontSize;

        });

        _html = _html.replace(/&nbsp;/g, " ");
        var _fontBits = Math.round(_containerWidth / _fontSize)
        var body = $('<div>' + _html + '</div>'),
            fontList = [];
        $('#compare').html(body.html()).append('<br>');
        var count = 0;
        if (body.find('b,i,u').length) {
            var cleanStop = false
            while (!cleanStop) {
                body.find('b,i,u').each(function() {
                    $(this).replaceWith('<font type="' + $(this)[0].tagName + '">' + $(this).html() + '</font>')
                });
                cleanStop = body.find('b,i,u').length == 0
            }
        }
        body.find('br').replaceWith('<font class="newLine"></font>')

        //$('#test').html('');
        body.find('p').each(function() {
            $(this).attr('root', true);
            var depend = $(this);
            var flag = true;
            while (flag) {
                if (depend.children('font').length == depend.find('font').length) {
                    if (depend.attr('root')) {
                        flag = false;
                    } else {
                        depend = depend.parent()
                    }
                }
                depend.children('font').each(function() {
                    var _child = $(this);
                    if (_child.find('font').length != 0) {
                        if (_child.find('font').length != _child.children('font').length) {
                            depend = _child;
                            return false;
                        } else {
                            splitAndJoin(_child);
                            $(this).children('font').each(function() {
                                var children = $(this),
                                    type = [];
                                !children.attr('color') && children.attr('color', _child.attr('color'));
                                _child.attr('type') && type.push(_child.attr('type'));
                                children.attr('type') && type.push(children.attr('type'));
                                type.length && children.attr('type', type.join(','));
                                _child.before(children.clone());
                                children.remove();
                            })
                            depend = _child.parent();
                            _child.remove();
                            return false;
                        }
                    }
                })
            }
            splitAndJoin(depend);
            // $('#test').append($(this).html());
            // $('#test').append('<br>')
            $(this).find('font').each(function() {
                fontList.push({
                    text: $(this).html(),
                    fontSize: _fontSize,
                    color: $(this).attr('color'),
                    type: $(this).attr('type'),
                    newLine: $(this).hasClass('newLine')
                });
            });
            fontList.push({
                newLine: true
            });
        });
        return parseText(fontList, _containerWidth, _fontBits, _align);
    }

    // 去除标签
    module.exports.strip_tags = function(input, allowed) {
        // 代码来源 http://phpjs.org/functions/strip_tags/
        allowed = (((allowed || '') + '')
                .toLowerCase()
                .match(/<[a-z][a-z0-9]*>/g) || [])
            .join('');
        var tags = /<\/?([a-z][a-z0-9]*)\b[^>]*>/gi,
            commentsAndPhpTags = /<!--[\s\S]*?-->|<\?(?:php)?[\s\S]*?\?>/gi;
        return input.replace(commentsAndPhpTags, '')
            .replace(tags, function($0, $1) {
                return allowed.indexOf('<' + $1.toLowerCase() + '>') > -1 ? $0 : '';
            });
    }
});