define(function(require, exports, module) {
    var config = require('../configure');
    var Cache = require('Cache');

    function sendRequest(url, reqType, resType, successCallback, errorCallback) {
        return $.ajax({
            type: reqType,
            url: url,
            dataType: resType,
            success: function(data) {
                typeof successCallback === 'function' && successCallback(data);
            },
            error: function() {
                typeof errorCallback === 'function' && errorCallback(data);
            }
        });
    }

    function notifyServer(apiType, apiName, ajaxType, query, jsonp) {
        var sid = Stone.get('sid');
        if (sid) query.sid = sid;
        return function(done) {
            sendRequest(url(apiType, apiName, query), ajaxType, jsonp ? 'jsonp' : 'json', function(data) {
                done(null, data);
            }, function(data) {
                console.error(data);
            });
        }
    }

    function url(type, name, params) {
        var query = [];
        if (_.isObject(params)) {
            for (var i in params) {
                query.push(i + '=' + params[i]);
            }
            params = query.join('&')
        }
        var url = '';
        switch (type) {
            // case 'api':
            //     url = config.host + ':' + config.apiPort + "/api/" + name + '/' + params;
            //     break;
            case 'file':
                // params 可配置指定后缀名，一般默认为空
                var ext = !_.isUndefined(params) ? ('/' + name + '.' + params) : '';
                url = config.resourceHost + '/resource' + ext + '?resid=' + name;
                break;
            case 'editor':
                url = config.host + "/editor/" + name + '/' + params;
                break;
            case 'audio_file':
                name = arguments.length == 1 ? '' : name;
                url = config.resourceHost + '/resource.mp3?resid=' + name;
                break;
            case 'script':
                name = arguments.length == 1 ? '' : name;
                url = config.resourceHost + '/getPlay.json?' + params;
                break;
        }
        return url;
    }

    module.exports.notifyServer = notifyServer;

    function htmlDecode(value) {
        if (value) {
            // console.log($('<div />').html(value).text())
            return $('<div />').html(value).text();
        } else {
            return '';
        }
    }

    module.exports.fileUrl = function(_fileId) {
        return url('file', _fileId);
    }

    module.exports.audioUrl = function(_fileId, flag) {
        return url('audio_file', _fileId);
    }

    module.exports.parse_path = function(id, path, size) {
        size = size && size.split('*')
        if (_.isUndefined(path)) {
            return url('file', id + (_.isUndefined(size) ? '' : ('&w=' + size[0] + '&h=' + size[1])));
        } else {
            return path;
        }
    }

    module.exports.parse_ext_path = function(id, ext) {
        return url('file', id, ext);
    }

    module.exports.getJSON = function(url, callback) {
        return sendRequest(url, 'GET', 'json', function(data) {
            typeof callback === 'function' && callback(data);
        });
    }

    // module.exports.getGameInit = function(_gameId, callback) {
    //     var queryString = {};
    //     queryString.gameId = _gameId;
    //     var sid = Stone.get('sid');
    //     if (sid) queryString.sid = sid;
    //     return sendRequest(url('editor', 'global', queryString), 'GET', 'json', function(data) {
    //         typeof callback === 'function' && callback(data.data);
    //     });
    // }

    var getGameScript = function(_gameId, callback) {
        var querystring = {};
        querystring.gameId = _gameId;
        var sid = Stone.get('sid'),
            version = request('version'),
            id = request('id');
        if (sid) querystring.sid = sid;
        if (typeof id != 'undefined' && id != '') {
            var urlTemp = url('script', 'getPlay', {
                id: id
            });
            if (request('v') == 'last') {
                urlTemp += '&version=lastest';
            };
            return sendRequest(urlTemp, 'GET', 'html', function(data) {
                var htmlData = htmlDecode(data);
                // htmlData = htmlData.replace(/\\\"/g, "\"");
                // console.log('htmlData===>',htmlData);
                var jsonData = JSON.parse(htmlData);
                var gameId = jsonData.global.gameId;
                store.set('gameid', gameId);
                typeof callback === 'function' && callback(jsonData);
            });
        } else {
            if (typeof version != 'undefined' && version != '') {
                var query = {
                    version: version,
                    gameid: _gameId
                };
                // return sendRequest(url('script', 'getPlay', query), 'GET', 'html', function(data) {
                //     var htmlData = htmlDecode(data);
                //     htmlData = htmlData.replace(/\\"/g, "\"");
                //     // console.log('htmlData===>',htmlData);
                //     var jsonData = JSON.parse(htmlData);
                //     // data['data'] = jsonData;
                //     typeof callback === 'function' && callback({
                //         data: jsonData
                //     });
                // });
            } else {
                var postData = {
                    gameId: _gameId,
                    type: 'plain'
                };
                // return postPhpData(config.host + "/Home/Game/preview/gameId/"+ _gameId +"type/plain"
                // return postPhpData(config.host + "/Plaintool/Tree/getScenesInGame", postData,'GET', 'json', function(data) {
                return postPhpData(config.host + "/Api/Play/preview", postData, 'GET', 'json', function(data) {
                    typeof callback === 'function' && callback(data);
                });
            }
        }
    }

    module.exports.getGameScript = getGameScript;

    module.exports.getGameInit = function(_gameId, callback) {
        if (typeof _gameId == 'undefined' || _gameId == '' || _gameId == null) {
            _gameId = store.get('gameid');
        };
        var queryString = {};
        queryString.gameId = _gameId;
        var sid = Stone.get('sid');
        if (sid) queryString.sid = sid;
        var postData = {
            gameId: _gameId
        }
        return postPhpData(config.host + "/Api/Play/getGlobal", postData, 'POST', 'json', function(data) {
            typeof callback === 'function' && callback(data.msg);
        });
    }

    module.exports.getGameId = function(id, callback) {
        var queryString = {};
        queryString.script_id = id;
        var sid = Stone.get('sid');
        if (sid) queryString.sid = sid;
        return sendRequest(url('api', 'game/getGameId', queryString), 'GET', 'json', function(data) {
            typeof callback === 'function' && callback(data.gameid);
        });
    }

    module.exports.saveRecord = function(data, callback) {
        var postData = {
            gameid: Cache.getGameid(),
            userid: Cache.simple('userId'),
            type: type,
            data: data
        };
        var url = config.host + "/Home/SaveDocument/save";
        return postPhpData(url, postData, 'POST', json, function(data) {
            typeof callback === 'function' && callback(data.msg);
        });
    }

    function postPhpData(url, postData, reqType, resType, successCallback, errorCallback) {
        return $.ajax({
            type: reqType,
            url: url,
            dataType: resType,
            data: postData,
            success: function(data) {
                if (resType == 'json' && _.isString(data)) data = JSON.parse(data);
                if (typeof successCallback === 'function') successCallback(data);
            },
            error: function(data) {
                if (resType == 'json' && _.isString(data)) data = JSON.parse(data);
                if (typeof errorCallback === 'function') errorCallback(data);
            }
        });
    }

})