/*
    类型： 资源模块
    方法：load 加载一个资源
         loadList 加载一个资源列表
         get 获取资源
         getAll 根据资源类型获取资源
*/
define(function(require, exports, module) {
    var resourceList = {
            'audio': {},
            'image': {}
        },
        resourcePathList = [];

    var _eventProxy = {};
    var totalMusicAttr = {};
    module.exports.checkSoundLoaded = function(resid){
        return resourceList['audio'][resid] ? true : false;
    }


    // 保存数据
    var push = function(_resourceType, _resourceId, _resourceBody) {
        if (_.isUndefined(resourceList[_resourceType]))
            return false;
        resourceList[_resourceType][_resourceId] = _resourceBody;
        // resourceList[_resourceType][_resourceId] = _resourceId;
    }

    // 加载图片
    var loadImage = function(_imageId, _imagePath, _imageAttr, _callback) {
        if (_.includes(resourcePathList, _imagePath)) {
            typeof _callback === 'function' && _callback(false);
            return false;
        }
        var image = new Image();
        image.crossOrigin = '*';
        image.onload = function() {
            push('image', _imageId, image);
            typeof _callback === 'function' && _callback(image);
        }
        image.onerror = function() {
            console.error('Image load error:' + _imagePath);
            typeof _callback === 'function' && _callback(false);
        }
        image.src = _imagePath;

    }

    // 加载音频
    var loadAudio = function(_audioId, _audioPath, _audioAttr, _callback) {
        if (_.includes(resourcePathList, _audioPath))
            return typeof _callback === 'function' && _callback(false);

        /* 新的加载方法 CreateJS Preloadjs */
        var preload = new createjs.LoadQueue();
        preload.installPlugin(createjs.Sound);
        // 监听进度事件
        preload.on('progress', function(progress) {
            typeof _taskCallback == 'function' && _taskCallback(progress.progress);
        })

        // 监听结束事件
        preload.on('complete', function() {
            typeof _callback === 'function' && _callback();
        }, this);

        // 监听文件加载事件
        preload.on('fileload', function(e) {
            var info = parseLoadResult(e);
            if (info.type == 'sound')
                _eventProxy.trigger('SOUND_PUSH', info.id, _audioAttr);
            var type = info.type == 'sound' ? 'audio' : 'image';
            push(type, info.id, info.result);
        });

        // 文件加载
        preload.loadManifest([{id:_audioId,src:_audioPath}]);
    }

    var loadMethod = {
        image: loadImage,
        audio: loadAudio
    }

    function cleanList(list) {
        var _depend = {},
            out = [];
        _.each(list, function(elem) {
            if (_.isUndefined(_depend[elem.id])) {
                _depend[elem.id] = 0;
                out.push(elem);
            }
        });
        return out;
    }

    function makePreloadList(list) {
        list = cleanList(list);
        var out = [];
        _.each(list, function(elem) {
            var tmp = {
                id: elem.id,
                src: elem.path,
                type: elem.type
            };
            if (elem.type == 'audio') {
                tmp.type = 'sound'
            } else {
                tmp.crossOrigin = '*';
            }
            out.push(tmp);
        });
        return out;
    }

    // 获取列表元素属性
    function makeListAttrs(list) {
        // var out = totalMusicAttr;
        _.each(list, function(elem) {
            totalMusicAttr[elem.id] = elem.attr;
        });
        return totalMusicAttr;
    }

    module.exports.destorySound = function(soundId){
        if (resourceList['audio'][soundId]) {
            resourceList['audio'][soundId] = {};
        }
    }

    function parseLoadResult(_result) {
        var result = {};
        // 结果对象
        result.result = _result.result;
        // 属性
        _.each(_result.item, function(elem, key) {
            result[key] = elem;
        });
        // 结果对象尺寸
        result.size = !_.isUndefined(_result.rawResult) ? _result.rawResult.size : null;
        // 结果对象类型
        result.mime = !_.isUndefined(_result.rawResult) ? _result.rawResult.type : null;
        //    console.log(_result.rawResult)
        return result;
    }

    // 加载资源
    module.exports.load = function(_type, _id, _path, _attr, _callback) {
        if (!_.isUndefined(loadMethod[_type])) {
            loadMethod[_type].call(this, _id, _path, _attr, _callback);
        }
    }

    // 加载一个资源列表
    module.exports.loadList = function(_resourceList, _callback, _taskCallback) {
        /* 新的加载方法 CreateJS Preloadjs */
        var preload = new createjs.LoadQueue();
        // console.log(_resourceList);
        _resourceAttrs = makeListAttrs(_resourceList);
        _resourceList = makePreloadList(_resourceList);
        preload.installPlugin(createjs.Sound);
        // 监听进度事件
        preload.on('progress', function(progress) {
            typeof _taskCallback == 'function' && _taskCallback(progress.progress);
        })

        // 监听结束事件
        preload.on('complete', function() {
            // preload.removeAll();
            typeof _callback === 'function' && _callback();
        }, this);

        // 监听文件加载事件
        preload.on('fileload', function(e) {
            var info = parseLoadResult(e);
            // console.log(info);
            if (info.type == 'sound')
                _eventProxy.trigger('SOUND_PUSH', info.id, _resourceAttrs[info.id]);
            var type = info.type == 'sound' ? 'audio' : 'image';
            push(type, info.id, info.result);
        });

        // 文件加载
        preload.loadManifest(_resourceList);
    }



    // 根据资源Id获取资源，需带上资源类型，默认取图片资源
    module.exports.get = function(_resourceId, _resourceType) {
        if (_.isUndefined(_resourceType) && _.isUndefined(resourceList.image[_resourceId])) {
            return false;
        }
        _resourceType = _.isUndefined(_resourceType) ? 'image' : _resourceType;
        return resourceList[_resourceType][_resourceId];
    }

    // 根据类型获取所有资源
    module.exports.getAll = function(_resourceType) {
        return resourceList[_resourceType];
    }

    // 事件处理对象
    module.exports.eventProxy = function(_ep) {
        _eventProxy = _ep;
    }

});
