// UI模块
define(function(require, exports, module) {
    var STAGE = {}; // 舞台对象
    var SPLASH = {}; // 闪屏
    var PROGRESS = {};
    var Cube = require('Cube'),
        helper = require('UI_helper_css'),
        netModule = require('Net'),
        Variate = require('Variate'),
        Cache = require('Cache');
    var Events = require('Events');
    var Container = {};

    var _eventProxy = {};

    var openFrom = request('type');
    var ua = navigator.userAgent.toLowerCase();

    var resizeStage = function() {
        var size = helper.getStageSize();
        STAGE.setAttr('realSize', {
            width: size.sWidth,
            height: size.sHeight
        });
        STAGE.size(size);
        Container.css(size);
    };

    var Layers = ['SPLASH'],
        LAYERS = {};

    // 初始化方法
    module.exports.init = function(_container, UIDATA) {
        // 创建舞台
        STAGE = Cube.create().Stage({
            container: _container
        });
        Container = $('#' + _container);
        //调整舞台尺寸
        resizeStage();
        // 舞台隐藏
        STAGE.hide();
        // 设置到全局变量
        Stone.set('stage', STAGE);
        // 分配Layer
        var zIndex = 1;
        _.each(Layers, function(elem) {
            var _temp = $(sprintf('<div id="Layer_%s" class="Layer"></div>', elem));
            _temp.css({
                width: STAGE.width(),
                height: STAGE.height(),
                'z-index': zIndex++
            });
            Container.append(_temp);
            LAYERS[elem] = _temp;
        });
    };

    function getLayerById(_layerId) {
        if (_.isUndefined(LAYERS[_layerId])) return false;
        return LAYERS[_layerId];
    }

    module.exports.splash = function(_splash, _callback) {
        var _splahLayer = getLayerById('SPLASH');
        _splahLayer.hide();
        // _eventProxy.trigger('splashOver');
        // typeof _callback === 'function' && _callback(image);

        var image = new Image();
        image.onload = function() {
            var _splahLayer = getLayerById('SPLASH'),
                _image = $(image);
            _splahLayer.show();
            _image.css({
                width: _splahLayer.width(),
                height: _splahLayer.height(),
                opacity: 0
            });
            _splahLayer.append(_image);
            _image.animateCSS('fadeIn', {
                duration: 500,
                callback: function() {
                    _image.css('opacity', 1);
                    _image.animateCSS('fadeOut', {
                        delay: 800,
                        duration: 1000,
                        callback: function() {
                            _splahLayer.remove();
                            _eventProxy.trigger('splashOver');
                            if (typeof _callback === 'function') _callback(image);
                        }
                    });
                }
            });
        };
        image.onerror = function() {
            if (typeof _callback === 'function') _callback(false);
        };
        image.src = _splash;
    };

    module.exports.showStage = function() {
        return STAGE.show();
    };

    // 构造UI
    module.exports.build = function(uiSetting, callback) {
        Thenjs(function(cont) {
            return netModule.getJSON(uiSetting, function(data) {
                uiSetting = data;
                cont();
            });
        }).then(function(cont) {
            helper.init(STAGE, uiSetting);
            // 判断音乐按钮状态
            // if (store.get('music') == 'on') {
            //     setMusicButton('on');
            // }
            if (typeof callback === 'function') callback();
        });

    };

    module.exports.show = function(_groupId) {
        helper.getElem(_groupId).show();
    };

    module.exports.showList = function(_list, force) {
        if (_.isString(_list)) _list = _list.split(' ');
        _.each(_list, function(elem) {
            var g = helper.getElem(elem);
            if (!g) return;
            if (force) {
                g.show();
            } else if (g.is(':visible')) {
                g.hide();
            } else {
                g.show();
            }
            // 遮罩开关
            //var mask = g.getAttr('mask');
            //!_.isUndefined(mask) && (g.isVisible() ? helper.getGroup(mask).show() : helper.getGroup(mask).hide())

        });
        //getLayerById('UI').draw();
    };


    module.exports.hide = function(_groupId) {
        return helper.getElem(_groupId).hide();
    };

    module.exports.hideList = function(_list, force) {
        if (_.isString(_list)) _list = _list.split(' ');
        _.each(_list, function(elem) {
            var g = helper.getElem(elem);
            if (force) {
                g.hide();
            } else if (g.is(':visible')) {
                g.hide();
            } else {
                g.show();
            }
            // 遮罩开关
            //    var mask = g.getAttr('mask');
            //!_.isUndefined(mask) && (g.is(':visible')? helper.getGroup(mask).show() : helper.getGroup(mask).hide())
        });
        //getLayerById('UI').draw();
    };

    module.exports.selectStart = function(_callback){
        var top = ($('#container').height() - $('#selectStart').height())/2;
        var left = ($('#container').width() - $('#selectStart').width())/2;
        // var left = (100 - $('#selectStart').width()) * $('#container').width()/200;
        $('#tips-container').width($('#container').width());
        $('#tips-container').height($('#container').height());
        $('#tips-container').css({left: $('#container').offset().left, top: 0});
        $('#selectStart').css({top: top + 'px',left: left + 'px'});
        $('#tips-container').show();
        $('#selectStart .selectButton').off('click').on('click', function(){
            $('#tips-container').hide();
            _callback($(this).data('action'));
        });
    }

    var height = 0;

    function addActivity(_elem, type, callback) {
        var activity = helper.getElem('ACTIVITY').getElem().children('.cube-c'),
            DEVIDER = helper.getElem('DEVIDER'),
            showDevider = false,
            devider_clone,
            devider; 
        if(DEVIDER){
            showDevider = true;
            devider = helper.getElem('DEVIDER').getElem().children('.cube-c').clone();
        }

        _elem.find('.text').css('opacity', 0);
        // _elem.find('.text').hide();
        _elem.show();
        //    devider.css('opacity', 0);
        if(showDevider){
            devider_clone = devider.clone();
            devider_clone.addClass('infoText');
            devider_clone.css('opacity', 0);
        }
        _elem.addClass('infoText');

        // if (type == 'branch') activity.append(devider_clone);
        if(activity.find('.loading.cube-c').length > 0){
            activity.find('.loading.cube-c').replaceWith(_elem);
        }else{
            activity.append(_elem);
            _eventProxy.trigger('hideLoading');
            // activity.append(_elem);
        }
        var divheight = 0;
        _.each(_elem.find('.text').find('p'), function(elem, key){
            if(key < 2)
                divheight += _elem.find('.text').find('p').height();
        });
        if(divheight > 0){
            _elem.find('.text').height(divheight);
        } 
        _elem.find('.text').find('p').css('padding', '0 10px');
        _elem.find('.text').css('top', (_elem.find('.TEXT').height() - _elem.find('.text').height())/ 2 + 'px');
        if (type == 'branch' && showDevider)
            activity.append(devider_clone);

        //setTimeout(function() {
        //var animateName = type == 'branch' ? 'fadeIn' : 'fadeInDown';
        var animateName = 'fadeIn';

        height += _elem.height();
        if (type == 'branch' && showDevider) {
            height += devider.height() * 2;
        }
        store.set('closePass', false);
        _elem.find('.text').animateCSS(animateName, {
            callback: function() {
                this.css('opacity', 1);
                if (!showDevider && typeof callback === 'function'){
                    store.set('closePass', true);
                    callback();
                }
            }
        });
        if (type == 'message' && showDevider){
            _elem.find('.cube-p.TEXT').css({position:'relative'});
            devider_clone.css('opacity', 0);
            devider_clone.css('top', _elem.find('.cube-p.TEXT').css('top'));
            _elem.append(devider_clone);
        }
        if(showDevider){
            devider_clone.animateCSS(animateName, {
                callback: function() {
                    this.css('opacity', 1);
                    store.set('closePass', true);
                    if (!store.get('overView') && typeof callback === 'function') callback();
                }
            });
        }
        // devider.animateCSS(animateName, {
        //     duration: _duration,
        //     callback: function() {
        //         this.css('opacity', 1)
        //     }
        // });
        // $.easie(0.950, 0.250, 0.795, 0.035)
        if (type == 'branch'){
            var scrollTop = activity.scrollTop() + _elem.height();
            if(showDevider)
                scrollTop += devider_clone.height();
            activity.stop().animate({
                'scrollTop': scrollTop
                // 'scrollTop': height
            }, 400, $.easie(0.445, 0.050, 0.550, 0.950));
        }
    }

    function addMessage(_elem) {
        // var activity = helper.getElem('ACTIVITY').getElem().children('.cube-c');
        // activity.append(_elem);
        // _elem.children('*').css('opacity', 1);
        // _elem.show();
        // height += _elem.height();
        // activity.stop().animate({
        //     'scrollTop': height
        // }, 800, $.easie(0.950, 0.250, 0.795, 0.035));
    }

    function playMusic(music, type){
        if(/iphone|ipad|ipod|iOS/.test(ua) && openFrom == 'box'){
            // if (/android/.test(ua)) {
                
            // } else {
                _eventProxy.trigger('playMusicInBox', music.resid, type);
            // }
        } else {
            _eventProxy.trigger('SOUND_PLAY', music.resid);
        }
    }

    function closeTempMusic(music, type){
        if(/iphone|ipad|ipod|iOS/.test(ua) && openFrom == 'box'){
            // if (/android/.test(ua)) {
                
            // } else {
                _eventProxy.trigger('closeTempMusicInBox', type);
            // }
        } else {
            _eventProxy.trigger('closeTempMusic', type);
        }
    }

    function closeBGM(type){
        if(/iphone|ipad|ipod|iOS/.test(ua) && openFrom == 'box'){
            _eventProxy.trigger('closeTempMusicInBox', 'music');
        } else {
            _eventProxy.trigger('SOUND_BACKGROUND', 'music');
        }
    }

    function timeFunc(_scene, _callback) {
        var messageElem = helper.getElem('MESSAGE');
        var $messageTextElem = messageElem.findOne('TEXT').getElem().clone();
        var $messageElem = messageElem.getElem().children('.cube-c').clone();
        if(_scene.textAlign && _scene.textAlign !== '')
            $messageTextElem.css('text-align', _scene.textAlign);
        // var content = Variate.showVariate(_scene.content);
        // $messageTextElem.children('.cube-c').html(content);
        $messageTextElem.children('.cube-c').html(_scene.content);
        $messageElem.find('.TEXT').replaceWith($messageTextElem);

        // if(_scene.music && _scene.music.resid){
        //     _eventProxy.trigger('SOUND_PLAY', _scene.music.resid);
        // } else {
        //     _eventProxy.trigger('SOUND_BACKGROUND', 'music');
        // }
        if(_scene.music && _scene.music.resid){
            // _eventProxy.trigger('playMusicInBox', musicId, type);
            playMusic(_scene.music, 'music');
        } else {
            closeBGM('music');
            // _eventProxy.trigger('SOUND_BACKGROUND', 'music');
        }
        if(_scene.sound && _scene.sound.resid){
            playMusic(_scene.sound, 'sound');
        } else {
            closeTempMusic(_scene.sound, 'sound');
        }
        Cache.setRenderScene(_scene);
        addActivity($messageElem, 'message', function() {
            Cache.del('taskTime');
            if (typeof _callback === 'function') _callback();
        });
    }

    module.exports.message = function(_scene, _callback) {
        var _delay = _scene.delay;
        var timestamp = Math.round(new Date().getTime() / 1000);
        var delaytime = _delay.split('+'),
            delay;
        if (delaytime.length < 2) delaytime = [2, 's'];
        switch (delaytime[1]) {
            case "m":
                delay = delaytime[0] * 60;
                break;
            case "h":
                delay = delaytime[0] * 3600;
                break;
            default:
                delay = delaytime[0] * 1;
                break;
        }
        if(!Events.next_flag)
            return false;
        var taskTime = Cache.get('taskTime') * 1,
            diffTime = taskTime ? (taskTime > timestamp ? (taskTime - timestamp) : 1) : delay;
        if (!taskTime) Cache.set('taskTime', timestamp + delay);
        // timeFunc(_scene, _callback);
        if (diffTime === 0) {
            timeFunc(_scene, _callback);
        } else {
            var timer = new Timer(),
                duration = diffTime;
            store.set('closePass', false);
            // _eventProxy.trigger('showLoading');
            timer.on('onend', function() {
                store.set('closePass', true);
                timeFunc(_scene, _callback);
            });

            timer.start(diffTime);
        }

    };

    // 清除列表
    module.exports.clearList = function() {
        Cache.clearRecordSceneList();
        Cache.clearRecordScene();
        Cache.del('taskTime');
        helper.getElem('ACTIVITY').getElem().children('.cube-c').find('.infoText').remove();
    };


    module.exports.messageList = function(_list) {
        var messageElem = helper.getElem('MESSAGE'),
            $messageTextElem = messageElem.findOne('TEXT').getElem().clone(),
            $messageElem = messageElem.getElem().children('.cube-c').clone();
        _.each(_list, function(elem) {
            $messageTextElem = $messageTextElem.clone();
            $messageElem = $messageElem.clone();
            $messageTextElem.children('.cube-c').html(elem.content);
            $messageElem.children('.TEXT').replaceWith($messageTextElem);
            $messageElem.addClass('infoText');
            addMessage($messageElem);
        });

    };

    //处理和显示选项
    module.exports.branch = function(_scene, _branchList, _callback) {
        var branchCount = 0,
            branchHeight = 0;
        var branchElem = helper.getElem('BRANCH'),
            $branchElem = branchElem.getElem().children('.cube-c').clone();
            // $branchElem.css('height', 'auto');
        // 分支选项按钮 绑定事件
        _.each(_branchList, function(elem, key) {
            var buttonElem = helper.getElem('BUTTON'),
                $buttonTextElem = buttonElem.findOne('TEXT').getElem().clone(),
                $buttonElem = buttonElem.getElem().clone();
            $buttonElem.data('targetId', _branchList[key].id);

            $buttonTextElem.children('.cube-c').html(_branchList[key].text);
            $buttonTextElem.children('.cube-c').html($buttonTextElem.children('.cube-c').text());
            $buttonElem.find('.TEXT').replaceWith($buttonTextElem);
            $buttonElem.off('tap click').on('tap click', function() {
                $(this).off('tap click');
                $(this).siblings().off('tap click');
                helper.setBackground(buttonElem.getAttr('active'), $buttonElem.children('.cube-c'));
                helper.setBackground(buttonElem.getAttr('visited'), $buttonElem.siblings().children('.cube-c'));
                $buttonTextElem.css('color', buttonElem.getAttr('active_color'));
                _eventProxy.trigger('clickBranch');
                var next = _branchList[key].id;
                var numericalTo = Variate.calcByScene(_scene.sceneId, false);
                next = numericalTo ? numericalTo : next;
                _eventProxy.trigger('prepare', next);
            });
            branchHeight = branchHeight + $buttonElem.height() + 5;
            $buttonElem.css('top', ($buttonElem.height() + 5) * branchCount + 'px');
            // $buttonElem.show();
            if(branchCount == 0){
                $branchElem.find('.BUTTON').eq(0).replaceWith($buttonElem);
            }else {
                $branchElem.append($buttonElem);
            }
            // $branchElem.append($buttonElem);
            branchCount++;
        });

        $branchElem.height(branchHeight);
        // $branchElem.find('.cube_bg').height(branchHeight / helper.getScale());
        if(_scene.music && _scene.music.resid){
            _eventProxy.trigger('SOUND_PLAY', _scene.music.resid);
        } else {
            _eventProxy.trigger('SOUND_BACKGROUND', 'music');
        }
        if(_scene.sound && _scene.sound.resid){
            _eventProxy.trigger('SOUND_PLAY', _scene.sound.resid);
        } else {
            closeTempMusic(_scene.sound, 'sound');
        }
        _eventProxy.trigger('hideLoading');
        addActivity($branchElem, 'branch');
    };

    //显示加载音乐的动画
    module.exports.showLoading = function(){
        var activity = helper.getElem('ACTIVITY').getElem().children('.cube-c'),
            loadingElem = helper.getElem('MESSAGE'),
            $loadingElem = loadingElem.getElem().children('.cube-c').clone();
        $loadingElem.addClass('loading');
        $loadingElem.find('.cube-p').remove();
        $loadingElem.append($('<div class="cube-c"></div>'));
        activity.append($loadingElem);
        activity.stop().animate({
            'scrollTop': activity.scrollTop() + $loadingElem.height() + 5
        }, 400, $.easie(0.445, 0.050, 0.550, 0.950));
    }

    //去除加载音乐的动画
    module.exports.hideLoading = function(){
        var $loadingElem = $('.loading.cube-c');
        $loadingElem.remove();
    }

    module.exports.setPlotTrace = function(_list) {
        var setContainer = helper.getElem('TRACE_CONTAINER'),
            setBox = helper.getElem('TRACE_BOX');
        var $setContainer = setContainer.getElem().children('.cube-c'),
            $setBox = setBox.getElem();
        $setContainer.html('');
        _.each(_list, function(elem, key) {
            var _setbox = $setBox.clone();
            _setbox.show();
            _setbox.find('.TEXT').children('.cube-c').html(elem.blockName);
            _setbox.off('click').on('click', function(e) {
                e.preventDefault();
                _eventProxy.trigger('plotTrace', elem.id);
            });
            $setContainer.append(_setbox);
            _setbox.css('top', 1.2 * key * _setbox.height());
        });
    };

    module.exports.setPlotOverview = function(_list, _index) {
        //        if (_list.length < _index * 5) return;
        var setContainer = helper.getElem('OVERVIEW_CONTAINER'),
            setBox = helper.getElem('OVERVIEW_BOX');
        var $setContainer = setContainer.getElem().children('.cube-c'),
            $setBox = setBox.getElem();
        $setContainer.html('');
        _.each(_list, function(elem, key) {
            var _setbox = $setBox.clone();
            _setbox.show();
            _setbox.find('.TEXT').children('.cube-c').html(elem.name);
            _setbox.off('click').on('click', function(e) {
                e.preventDefault();
                _eventProxy.trigger('gotoOverview', elem.id);
            });
            $setContainer.append(_setbox);
            _setbox.css('top', 1.3 * key * _setbox.height());
        });
    };

    function setMusicButton(_switch) {
        var setMusicButton = helper.getElem('BUTTONM');

        $setMusicButton = setMusicButton.getElem();
        $setMusicText = setMusicButton.findOne('TEXT').getElem();
        if (_switch == 'on') {
            helper.setBackground(setMusicButton.getAttr('active'), $setMusicButton.children('.cube-c'));
            $setMusicText.css('color', setMusicButton.getAttr('active_color'));
            $setMusicText.children('.cube-c').html('打开音乐');
        } else {
            helper.setBackground(setMusicButton.getAttr('background'), $setMusicButton.children('.cube-c'));
            $setMusicText.css('color', setMusicButton.getAttr('active_color'));
            $setMusicText.children('.cube-c').html('关闭音乐');
        }
    }

    module.exports.setMusic = function() {
        var _musicSwitch = store.get('music');
        if (!_musicSwitch) _musicSwitch = 'off';
        _eventProxy.trigger('SETTING_MUSIC', _musicSwitch);
        _musicSwitch = _musicSwitch == 'on' ? 'off' : 'on';
        store.set('music', _musicSwitch);
        setMusicButton(_musicSwitch);
    };

    module.exports.setSttingTitle = function(_text) {
        var title = helper.getElem('SETTING_TITLE').getElem();
        title.children('.cube-c').html(_text);
    };

    // 事件处理对象
    module.exports.eventProxy = function(_ep) {
        _eventProxy = _ep;
        helper.eventProxy(_ep);
        _eventProxy.on('closeBGM', function(){
            closeBGM('music');
        });
    };

});
