// UI模块
define(function(require, exports, module) {
    var mainLayer = {},
        Elems = {},
        Images = {},
        setting = {},
        baseWidth = 720,
        baseImage = {},
        meta = {},
        scale = 1,
        metaScale = 0,
        uiDATA = {},
        Cube = require('Cube'),
        Cache = require('Cache');

    var _eventProxy = {};

    // 尺寸适配
    var sizeFit = function(_opts) {
        _.each(['x', 'y', 'w', 'h', 'minHeignt', 'fontSize','lineHeight'], function(elem) {
            if (!_.isUndefined(_opts[elem]))
                _opts[elem] = _opts[elem] * scale;
        });
        return _opts;
    };

    module.exports.sizeFit = sizeFit;

    // 获取组
    var getElem = function(_elemId) {
        return _.isUndefined(Elems[_elemId]) ? false : Elems[_elemId];
    };

    module.exports.getElem = getElem;

    module.exports.init = function(container, data) {
        scale = container.width() / baseWidth;
        mainLayer = container;
        uiDATA = data;
        var style = sprintf('<style>.cube_bg, .cube-c.loading .cube-c, #selectStart{transform:scale(%s);-webkit-transform:scale(%s)} </style>', scale, scale);
        $('body').append(style);
        makeUI(data, 'MAINUI');
    };

    function makeUI(_data, _env, _key, _flag) {
        if (_.isString(_env)) {
            parseStopFlag = _env;
            _env = {};
        }

        _flag = _key == 'reDrawUI' ? true : (_.isUndefined(_flag) ? false : _flag);
        var Parent = mainLayer;
        _env = _.extend({
            w: 720,
            h: 1280,
            x: 0,
            y: 0
        }, _env);

        if (!_.isUndefined(_key)) Parent = mainLayer.findOne(_key);

        _.each(_data, function(elem, key) {
            // if(key == 'ACTIVITY_CONTAINER'){
            //     if(Cache.getDefaultDirection()){
            //         elem.env.textAlign = Cache.getDefaultDirection();
            //     }
            //     if(Cache.getDefaultColor()){
            //         elem.env.color = Cache.getDefaultColor();
            //     }
            // }
            var _elem = makeElem(key, elem.env, _flag);
            !_.isUndefined(elem.event) && makeEvent(elem.event, elem.env.background, _elem, _flag);
            Parent.add(_elem);
            if (!_.isUndefined(elem.son)) {
                return makeUI(elem.son, elem.env, key, _flag);
            }
        });

        if (_key == parseStopFlag)
            mainLayer.draw();
    }

    function makeElem(_id, _env, flag) {
        _env = _.extend({
            w: 720,
            h: 1280,
            x: 0,
            y: 0,
            fontSize: 32,
            lineHeight:32,
            id: _id,
            overflow:'hidden'
        }, _env);

        _env = sizeFit(_env);
        var elem = Cube.create().Elem(_env);
        // 添加到Groups变量
        if (_.isUndefined(Elems[_id])) Elems[_id] = elem;
        return elem;
    }

    // 构造事件
    var clickEvent = {
        target: '',
        times: 0
    }

    function makeEvent(_event, _oriSkin, elem, flag) {
        if (!_.isUndefined(_event.hover)) {
            elem.off('mouseover touchstart').on('mouseover touchstart', function(e) {
                //    e.preventDefault();
                e.stopPropagation();
                change(_event.hover.background, elem);
            });

            elem.off('mouseleave touchend').on('mouseleave touchend', function(e) {
                //    e.preventDefault();
                e.stopPropagation();
                change(_oriSkin, elem);
            });
        }

        // 绑定click
        if (!_.isUndefined(_event.click)) {
            var show = _.isString(_event.click.show) ? _event.click.show.split(' ') : [],
                hide = _.isString(_event.click.hide) ? _event.click.hide.split(' ') : [];
            elem.off('click tap').on('click tap', function(e) {
                e.preventDefault();
                e.stopPropagation();
                if (clickEvent.target != elem.getId()) {
                    clickEvent.target = elem.getId();
                    clickEvent.times = 0;
                } else {
                    clickEvent.times++;
                }
                _eventProxy.trigger('show', clickEvent.times % 2 ? hide : show, true);
                _eventProxy.trigger('hide', clickEvent.times % 2 ? show : hide, true);
                !_.isUndefined(_event.click.event) && _eventProxy.trigger(_event.click.event, elem);
            });
        }
    }

    function change(_background, elem) {
        elem.setBackground(_background);
    }

    module.exports.setBackground = function(_background, elem) {
        var background = elem.children('.cube_bg');
        background.removeAttr('class');
        background.addClass(_background + ' sprite cube_bg');
    };

    // 获取舞台的尺寸，包括在旋转状态下的计算
    module.exports.getStageSize = function(_rotateStatus) {
        var pageSize = get_page_size(),
            stageSize = {
                width: pageSize.width,
                height: pageSize.height
            };

        if (stageSize.width > stageSize.height) {
            stageSize.width = stageSize.height * 9 / 16;
        } else if (stageSize.width < stageSize.height) {
            if (stageSize.width / stageSize.height > 9 / 16) {
                stageSize.width = stageSize.height * 9 / 16;
            } else {
                stageSize.height = stageSize.width * 16 / 9;
            }
        }

        stageSize.width = Math.ceil(stageSize.width);
        stageSize.height = Math.ceil(stageSize.height);

        if (stageSize.width > 720 && stageSize.height > 1280) {
            stageSize.width = 720;
            stageSize.height = 1280;
        }

        var _rotateStageSize = {
            width: stageSize.height,
            height: stageSize.width
        };

        var a = _.extend({
            x: _rotateStatus ? _rotateStageSize.width : 0,
            sWidth: stageSize.width,
            sHeight: stageSize.height
        }, _rotateStatus ? _rotateStageSize : stageSize);
        return a;
    };


    // 将RGB颜色转换成16进制
    // 参考来源 http://www.zhangxinxu.com/wordpress/2010/03/javascript-hex-rgb-hsl-color-convert/
    module.exports.RGB2Hex = function(_color) {
        var that = _color;
        if (/^(rgb|RGB)/.test(that)) {
            var aColor = that.replace(/(?:\(|\)|rgb|RGB)*/g, '').split(',');
            var strHex = '#';
            for (var i = 0; i < aColor.length; i++) {
                var hex = Number(aColor[i]).toString(16);
                if (hex === '0')
                    hex += hex;
                strHex += hex;
            }
            if (strHex.length !== 7)
                strHex = that;
            return strHex;
        } else if (/^#([0-9a-fA-f]{3}|[0-9a-fA-f]{6})$/.test(that)) {
            var aNum = that.replace(/#/, '').split('');
            if (aNum.length === 6) {
                return that;
            } else if (aNum.length === 3) {
                var numHex = '#';
                for (var i = 0; i < aNum.length; i += 1) {
                    numHex += (aNum[i] + aNum[i]);
                }
                return numHex;
            }
        } else {
            return that;
        }
    };

    // 将16进制转换成RGB颜色
    module.exports.Hex2RGB = function(_color, _toArray) {
        var sColor = _color.toLowerCase();
        if (sColor && /^#([0-9a-fA-f]{3}|[0-9a-fA-f]{6})$/.test(sColor)) {
            if (sColor.length === 4) {
                var sColorNew = '#';
                for (var i = 1; i < 4; i += 1) {
                    sColorNew += sColor.slice(i, i + 1).concat(sColor.slice(i, i + 1));
                }
                sColor = sColorNew;
            }
            // 处理六位的颜色值
            var sColorChange = []
            for (var i = 1; i < 7; i += 2) {
                sColorChange.push(parseInt('0x' + sColor.slice(i, i + 2)));
            }
            return _toArray ? sColorChange : ('RGB(' + sColorChange.join(',') + ')');
        } else {
            return sColor;
        }
    }

    module.exports.eventProxy = function(_ep) {
        _eventProxy = _ep;
    }

    module.exports.getScale = function(){
        return scale;
    }

});
